@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2017 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -1856,7 +1856,7 @@ static void _dom_document_schema_validate(INTERNAL_FUNCTION_PARAMETERS, int type
 	vptr = xmlSchemaNewValidCtxt(sptr);
 	if (!vptr) {
 		xmlSchemaFree(sptr);
-		php_error(E_ERROR, "Invalid Schema Validation Context");
+		zend_throw_error(NULL, "Invalid Schema Validation Context");
 		RETURN_FALSE;
 	}
 
@@ -1956,7 +1956,7 @@ static void _dom_document_relaxNG_validate(INTERNAL_FUNCTION_PARAMETERS, int typ
 	vptr = xmlRelaxNGNewValidCtxt(sptr);
 	if (!vptr) {
 		xmlRelaxNGFree(sptr);
-		php_error(E_ERROR, "Invalid RelaxNG Validation Context");
+		zend_throw_error(NULL, "Invalid RelaxNG Validation Context");
 		RETURN_FALSE;
 	}
 
@@ -2244,15 +2244,11 @@ PHP_METHOD(domdocument, registerNodeClass)
 
 	if (ce == NULL || instanceof_function(ce, basece)) {
 		DOM_GET_OBJ(docp, id, xmlDocPtr, intern);
-
-		if (dom_set_doc_classmap(intern->document, basece, ce) == FAILURE) {
-			php_error_docref(NULL, E_ERROR, "Class %s could not be registered.", ZSTR_VAL(ce->name));
-		}
+		dom_set_doc_classmap(intern->document, basece, ce);
 		RETURN_TRUE;
-	} else {
-		php_error_docref(NULL, E_ERROR, "Class %s is not derived from %s.", ZSTR_VAL(ce->name), ZSTR_VAL(basece->name));
 	}
-
+	
+	zend_throw_error(NULL, "Class %s is not derived from %s.", ZSTR_VAL(ce->name), ZSTR_VAL(basece->name));
 	RETURN_FALSE;
 }
 /* }}} */
