@@ -32,6 +32,9 @@
 #define MAY_BE_RC1                  (1<<27) /* may be non-reference with refcount == 1 */
 #define MAY_BE_RCN                  (1<<28) /* may be non-reference with refcount > 1  */
 
+#define MAY_HAVE_DTOR \
+	(MAY_BE_OBJECT|MAY_BE_RESOURCE \
+	|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_OBJECT|MAY_BE_ARRAY_OF_RESOURCE)
 
 #define DEFINE_SSA_OP_HAS_RANGE(opN) \
 	static zend_always_inline zend_bool _ssa_##opN##_has_range(const zend_op_array *op_array, const zend_ssa *ssa, const zend_op *opline) \
@@ -163,14 +166,15 @@ static zend_always_inline uint32_t _const_op_type(const zval *zv) {
 	} else if (Z_TYPE_P(zv) == IS_ARRAY) {
 		HashTable *ht = Z_ARRVAL_P(zv);
 		uint32_t tmp = MAY_BE_ARRAY;
+		zend_string *str;
+		zval *val;
 
 		if (Z_REFCOUNTED_P(zv)) {
 			tmp |= MAY_BE_RC1 | MAY_BE_RCN;
 		} else {
 			tmp |= MAY_BE_RCN;
 		}
-		zend_string *str;
-		zval *val;
+
 		ZEND_HASH_FOREACH_STR_KEY_VAL(ht, str, val) {
 			if (str) {
 				tmp |= MAY_BE_ARRAY_KEY_STRING;
@@ -262,6 +266,8 @@ void zend_func_return_info(const zend_op_array   *op_array,
                            int                    widening,
                            zend_ssa_var_info     *ret);
 
+int zend_may_throw(const zend_op *opline, zend_op_array *op_array, zend_ssa *ssa);
+
 END_EXTERN_C()
 
 #endif /* ZEND_INFERENCE_H */
