@@ -1,8 +1,8 @@
-/* 
+/*
 	+----------------------------------------------------------------------+
-	| PHP Version 5                                                        |
+	| PHP Version 7                                                        |
 	+----------------------------------------------------------------------+
-	| Copyright (c) 1997-2016 The PHP Group                                |
+	| Copyright (c) 1997-2018 The PHP Group                                |
 	+----------------------------------------------------------------------+
 	| This source file is subject to version 3.01 of the PHP license,      |
 	| that is bundled with this package in the file LICENSE, and is        |
@@ -29,10 +29,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#ifndef ZEND_ENGINE_2
-#error HEAD does not work with ZendEngine1 anymore
-#endif
-
 #include "ext/standard/dl.h"
 #include "ext/standard/file.h"
 #include "ext/standard/fsock.h"
@@ -186,10 +182,10 @@ static zend_module_entry *php_builtin_extensions[] = {
 /* }}} */
 
 #define EXTCOUNT (sizeof(php_builtin_extensions)/sizeof(zend_module_entry *))
-	
-PHPAPI int php_register_internal_extensions(TSRMLS_D)
+
+PHPAPI int php_register_internal_extensions(void)
 {
-	return php_register_extensions(php_builtin_extensions, EXTCOUNT TSRMLS_CC);
+	return php_register_extensions(php_builtin_extensions, EXTCOUNT);
 }
 
 /*
