@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2017 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -2095,6 +2095,12 @@ static int php_cli_server_dispatch_router(php_cli_server *server, php_cli_server
 {
 	int decline = 0;
 	zend_file_handle zfd;
+	char *old_cwd;
+
+	ALLOCA_FLAG(use_heap)
+	old_cwd = do_alloca(MAXPATHLEN, use_heap);
+	old_cwd[0] = '\0';
+	php_ignore_value(VCWD_GETCWD(old_cwd, MAXPATHLEN - 1));
 
 	zfd.type = ZEND_HANDLE_FILENAME;
 	zfd.filename = server->router;
@@ -2116,6 +2122,12 @@ static int php_cli_server_dispatch_router(php_cli_server *server, php_cli_server
 		}
 	} zend_end_try();
 
+	if (old_cwd[0] != '\0') {
+		php_ignore_value(VCWD_CHDIR(old_cwd));
+	}
+
+	free_alloca(old_cwd, use_heap);
+
 	return decline;
 }
 /* }}} */
@@ -2307,20 +2319,10 @@ static int php_cli_server_ctor(php_cli_server *server, const char *addr, const c
 
 	if (router) {
 		size_t router_len = strlen(router);
-		if (!IS_ABSOLUTE_PATH(router, router_len)) {
-			_router = pemalloc(server->document_root_len + router_len + 2, 1);
-			if (!_router) {
-				retval = FAILURE;
-				goto out;
-			}
-			snprintf(_router,
-				server->document_root_len + router_len + 2, "%s%c%s", server->document_root, DEFAULT_SLASH, router);
-		} else {
-			_router = pestrndup(router, router_len, 1);
-			if (!_router) {
-				retval = FAILURE;
-				goto out;
-			}
+		_router = pestrndup(router, router_len, 1);
+		if (!_router) {
+			retval = FAILURE;
+			goto out;
 		}
 		server->router = _router;
 		server->router_len = router_len;
